@@ -1,7 +1,7 @@
 #ifndef __ARES_INET_NTOP_H
 #define __ARES_INET_NTOP_H
 
-/* $Id: inet_ntop.h,v 1.3 2007-11-19 15:47:01 bagder Exp $ */
+/* $Id: inet_ntop.h,v 1.4 2008-09-24 16:43:12 yangtse Exp $ */
 
 /* Copyright (C) 2005 by Dominick Meglio
  *
@@ -18,7 +18,7 @@
  * without express or implied warranty.
  */
 
-#if defined(HAVE_INET_NTOP) && defined(HAVE_INET_NTOP_IPV6)
+#ifdef HAVE_INET_NTOP
 #define ares_inet_ntop(w,x,y,z) inet_ntop(w,x,y,z)
 #else
 const char *ares_inet_ntop(int af, const void *src, char *dst, size_t size);
